//
//  SSMasterViewController.m
//  Example
//
//  Created by Bertils mini on 2013-10-17.
//  Copyright (c) 2013-14 Sork Software. All rights reserved.
//
//  Added viewWillDisappear jan -14
//

#import "SSMasterViewController.h"
#import "SSDetailViewController.h"
#import "EditView.h"
#import "MovieTitle.h"

@interface SSMasterViewController () {
    NSMutableArray *_movies;
    NSMutableArray *_titles;
    NSUInteger selectedRow;
    CGFloat accessoryHeight;
    UIBarButtonItem *addButton;
}

@property (strong, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UITextField *titleField;
@property (weak, nonatomic) IBOutlet UIPickerView *genrePicker;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *tableViewTop;
@property (strong, nonatomic) IBOutlet EditView *editView;

@end

@implementation SSMasterViewController

#pragma mark - Lifecycle

- (void)awakeFromNib {
    self.preferredContentSize = CGSizeMake(320.0, 600.0);
    
    [super awakeFromNib];
}

- (void)viewDidLoad {
  [super viewDidLoad];
  
	// Do any additional setup after loading the view, typically from a nib
  self.navigationItem.rightBarButtonItem = self.editButtonItem;
  
  // Create a button for adding a movie table entry
  addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(addNewMovie:)];
  
  // Dig out a reference to the detail view controller
  self.detailViewController = (SSDetailViewController *)[[self.splitViewController.viewControllers lastObject] topViewController];
  
  // Create a storage for our movie titles. Should really be a set
  _titles = [[NSMutableArray alloc] init];
  
  // Consider some favorites - Toy Thriller, Dr Funnylove, Gulp Fission, White Swan, Taxi Customer
  //_titles = (NSMutableArray *)@[@"The Polishing", @"An Unusual Suspect", @"Love History", @"A Moose Story", @"The Family Joules"];
  
  // Define some genres for the flicks
  self.genres = @[@"Crime", @"Comedy", @"Documentary", @"Drama", @"Mystery", @"Sci-Fi", @"Thriller"];
  
  // Create a bunch of movies with genre
  MovieTitle *aTitle = [MovieTitle movieTitle:@"The Polishing" withGenre:6];
  [_titles addObject:aTitle];
  aTitle = [MovieTitle movieTitle:@"An Unusual Suspect" withGenre:0];
  [_titles addObject:aTitle];
  aTitle = [MovieTitle movieTitle:@"The Family Joules" withGenre:3];
  [_titles addObject:aTitle];
  
  // Show the first title
  selectedRow = 0;
  MovieTitle *movie = _titles[selectedRow];
  
  // Let the detail view know about the first title
  self.detailViewController.detailItem = movie;
}

// Because viewWillAppear is too early to check mark a cell
- (void)viewDidAppear:(BOOL)animated {
  [super viewDidAppear:animated];
  
  [self checkSelectedTitle];
}

// End editing when the portrait popover is dismissed
- (void)viewWillDisappear:(BOOL)animated {
  [super viewWillDisappear:animated];
  
	// End any ongoing editing if applicable
	if (self.editing) {
		[self setEditing:NO animated:NO];
	}
}

// Must add this to a plain view controller (replacing the original table view controller)
// Invoked when the user touches the Edit/Done button
- (void)setEditing:(BOOL)editing animated:(BOOL)animated {
  // Update the appearance of the Edit/Done button as appropriate
  [super setEditing:editing animated:animated];
  
	if (editing) {
		// Editing will start, show the Add button
		[self.navigationItem setLeftBarButtonItem:addButton animated:YES];
    
		// Show the edit accessory view with the title field and the picker
		// The animation completion block will update the content
    [self showAccessoryView];
	} else {
		// Editing will stop, hide the Add button
		[self.navigationItem setLeftBarButtonItem:nil animated:YES];
    
		// End any text editing, saving the changes, if any, in textFieldDidEndEditing
		[self.titleField resignFirstResponder];
    
		// Hide the accessory view
		[self hideAccessoryView];
    
    // Show the correct selection if this was changed during editing
    [self checkSelectedTitle];
	}
	// Actually change the table view to or from editing
  [self.tableView setEditing:editing animated:YES];
	
	// And update its content
	[self.tableView reloadData];
  
	// Remove the popover when editing ends in portrait orientation
	if (!editing) {
		if (self.detailViewController.masterPopoverController != nil) {
			[self.detailViewController.masterPopoverController dismissPopoverAnimated:YES];
		}
	}
}

#pragma mark - Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
  
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
  
  return _titles.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
  
  // Movie title for the cell
  MovieTitle *movie = _titles[indexPath.row];
  // Show its title and genre
  cell.textLabel.text = [movie movieTitle];
  cell.detailTextLabel.text = _genres[[movie movieGenre]];
  
  return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
  // Return NO if you do not want the specified item to be editable
  return YES;
}

// Must handle deleted titles here
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
  // This is the selected movie
  MovieTitle *movie = _titles[selectedRow];
  // With the title
  NSString *selectedTitle = [movie movieTitle];
  
  // Make sure this is a delete
  if (editingStyle == UITableViewCellEditingStyleDelete) {
    // Is it the selected row that is deleted?
    if (selectedRow == indexPath.row) {
      // Yes, get the row index to be deleted
			NSUInteger index = indexPath.row;
			// Try changing the selection to the next row
			index++;
			// Reset to the beginning if past the list of titles
			if (index >= [_titles count]) index = 0;
      
      // This will be the newly selected movie
      MovieTitle *movie = _titles[index];
      // Get its title
      selectedTitle = [movie movieTitle];
      
      // Let the detail view know about the new title
      self.detailViewController.detailItem = movie;
    }
    // Delete the title at the edited row from the register
    [_titles removeObjectAtIndex:indexPath.row];
    
    // Also get rid of its table view entry
    [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    
    // Make sure we have the correct index of the selection after a deletion
    NSUInteger index = [self indexForTitle:selectedTitle];
    // This will be the selection
    selectedRow = index;
    
    // Update the title field and the genre picker
    [self updateEditView];
    
    // Actually select a row for editing after the deletion of a title
		[self performSelector:@selector(selectEditedTitle) withObject:nil afterDelay:0.0];
		
		[self.tableView reloadData];
    
  } else if (editingStyle == UITableViewCellEditingStyleInsert) {
    // We do not use this for new additions
  }
}

// The user selected another table view row
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  // The new row index
  selectedRow = indexPath.row;
  
  // Let the detail view know about the change
  MovieTitle *movie = _titles[selectedRow];
  self.detailViewController.detailItem = movie;
  
  if (!self.editing) {
    // Not editing, just move the selection mark
    [self checkSelectedTitle];
    
		// iPad table views do not display a persistent selected state
		[tableView deselectRowAtIndexPath:indexPath animated:YES];
		
		// In portrait view, remove a popover after a selection
		if (self.detailViewController.masterPopoverController != nil) {
			[self.detailViewController.masterPopoverController dismissPopoverAnimated:YES];
		}
	} else {
		// Editing, select the newly selected row
		[self.tableView selectRowAtIndexPath:indexPath animated:YES scrollPosition:UITableViewScrollPositionNone];
    
		// Update the title field and the picker for the new row
		[self updateEditView];
	}
}

#pragma mark Table view support methods

// Find the storage index of a specific movie title
// Used by commitEditingStyle, textFieldDidEndEditing and addNewMove
- (NSUInteger)indexForTitle:(NSString *)aTitle {
  NSUInteger __block index = 0;
  
  // Block enumaration can be somewhat faster than a for/in fast enumeration...
  // And it offers an index making the inefficient indexOfObject: unnecessary :]
  [_titles enumerateObjectsUsingBlock:^(MovieTitle *movie, NSUInteger idx, BOOL *stop) {
    // Is this the sought movie title?
    NSString *aMovieTitle = [movie movieTitle];
    if ([aMovieTitle isEqualToString:aTitle]) {
      // Yes, return its index and stop the search
      index = idx;
      *stop = YES;
    }
  }];
  return index;
}

#pragma mark - Text field delegate

// Received when the delegate should process the Return/Done button
- (BOOL)textFieldShouldReturn:(UITextField *)theTextField {
	if (theTextField == self.titleField) {
		// Remove the keyboard and save the new/modified text
		[self.titleField resignFirstResponder];
	}
	return YES;
}

// Received when the keyboard hides. The edited title is always the selected one
- (void)textFieldDidEndEditing:(UITextField *)textField {
  // This is the current title
  MovieTitle *movie = _titles[selectedRow];
  NSString *selectedTitle = [movie movieTitle];
  
	// Has the edited movie title been changed?
	if (![selectedTitle isEqualToString:textField.text]) {
		// Yes, first make sure it is a valid title. Is the text field empty?
		if (0 == textField.text.length) {
			// Yes, replace the empty string with a unique title
			textField.text = [self newMovieTitle];
		}
    
    // Delete the changed title
    [_titles removeObjectAtIndex:selectedRow];
		
		// The new/modified title
		selectedTitle = textField.text;
    
    // Add a movie title with the text field title and the picker genre
    MovieTitle *movie = [self createMovieWithTitle:selectedTitle];
    [_titles addObject:movie];
    
    // Let the detail view know about the change
    self.detailViewController.detailItem = movie;
    
    // Sort the movie titles array alphabetically using this cool block thingy
    [_titles sortUsingComparator:^(MovieTitle *title1, MovieTitle *title2) {
      return [title1.movieTitle localizedCaseInsensitiveCompare:title2.movieTitle];
    }];
    
    // Find the index of the new/changed title in the sorted list
    NSUInteger index = [self indexForTitle:selectedTitle];
    
    // This is the new row selection
    selectedRow = index;
    
		// Run this operation after the table view update to select the edited title
		[self performSelector:@selector(selectEditedTitle) withObject:nil afterDelay:0.0];
    
		// Show the updated games list
		[self.tableView reloadData];
	}
}

#pragma mark - Picker view delegate methods

// The user choose a different picker row
- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component {
  // The newly chosen picker genre
  NSString *newGenre = self.genres[row];
  
  // This is the currently selected movie title
  MovieTitle *movie = _titles[selectedRow];
  
  // Update its genre
  movie.movieGenre = [self.genres indexOfObject:newGenre];
  
  // Let the detail view know about the change
  self.detailViewController.detailItem = movie;
  
  // Convert the selected table row number to an index path
  NSIndexPath *aPath = [NSIndexPath indexPathForRow:selectedRow inSection:0];
  
  // Run this operation after the table view update to select the edited title
  [self performSelector:@selector(selectEditedTitle) withObject:nil afterDelay:0.0];
  
  // Update the changed cell. Literals are cool!
  [self.tableView reloadRowsAtIndexPaths:@[aPath] withRowAnimation:UITableViewRowAnimationNone];
}

// The picker view text comes from the objects array
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component {
  
  return _genres[row];
}

#pragma mark Picker view data source methods

// This method determines the maximum picker wheel settings
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
	NSInteger numberOfRows = [_genres count];
  
  return numberOfRows;
}

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
  
	return 1;
}

#pragma mark - Sidebar support stuff

// Show the EditView accessory with animation, creating it lazily if needed
- (UIView *)showAccessoryView {
  if (!self.editView) {
    // Create an EditView from its own xib file
    [[NSBundle mainBundle] loadNibNamed:@"EditView" owner:self options:nil];
    
    // Configure the text field Return key
    self.titleField.returnKeyType = UIReturnKeyDone;
  }
  // Add the new view to the master container view
	[self.view addSubview:self.editView];
  
  // Autolayout stuff!
  // Remove the constraint that pins the table view to the top
  // This was originally created in the storyboard, or later, by our accessory removal method
  [self.view removeConstraint:self.tableViewTop];
  
  // This is the original table top, that is the bottom of the Nav bar
  id topGuide = self.topLayoutGuide;
  
  NSDictionary *views = @{ @"topGuide" : topGuide, @"editView" : self.editView, @"tableView" : self.tableView };
  
  // Add vertical constraints between the top and the two views
  NSArray *constraints = [NSLayoutConstraint
                          constraintsWithVisualFormat:@"V:[topGuide][editView][tableView]"
                          options:0
                          metrics:nil
                          views:views];
  [self.view addConstraints:constraints];
  
  // Get the first constraint in this set, the one between the topLayoutGuide and the edit view
  NSLayoutConstraint *topConstraint = [constraints firstObject];
  // Store it for a removal later
  self.tableViewTop = topConstraint;
  
  // Get the height of the edit view that might vary with localization etc
  CGRect bounds = self.editView.bounds;
  // Should use CGGeometry function that standardize the rectangle
  accessoryHeight = CGRectGetHeight(bounds);
  
  // Position the edit view above the table view, not yet visible
  topConstraint.constant = -accessoryHeight;
  
  // Add horisontal constraints for the new edit view
  constraints = [NSLayoutConstraint
                 constraintsWithVisualFormat:@"|[editView]|"
                 options:0
                 metrics:nil
                 views:views];
  [self.view addConstraints:constraints];
  
  // Layout the subviews before animating
  [self.view layoutIfNeeded];
  
  [UIView animateWithDuration:0.31 animations:^{
    // Animate the edit view into a visible position
    topConstraint.constant = 0.0;
    
    [self.view layoutIfNeeded];
  } completion:^(BOOL finished) {
    // Find and select the row for the selected title
    [self selectEditedTitle];
    
    // Update the text field and pickers
    [self updateEditView];
  }];
	return self.editView;
}

// Hide the edit view accessory with animation
- (void)hideAccessoryView {
  // Get our top constraint again
  NSLayoutConstraint *topConstraint = self.tableViewTop;
  
  // Animate the removal of the edit view
  [UIView animateWithDuration:0.31 animations:^{
    // Move the edit view outside again
    topConstraint.constant = -accessoryHeight;
    
    [self.view layoutIfNeeded];
  } completion:^(BOOL finished) {
    // Remove the now unwanted edit view with its constraints
    [self.editView removeFromSuperview];
  }];
  
  // Create and add a new top constraint for the table view
  id topGuide = self.topLayoutGuide;
  
  NSDictionary *views = @{ @"topGuide" : topGuide, @"tableView" : self.tableView };
  
  NSArray *constraints = [NSLayoutConstraint
                          constraintsWithVisualFormat:@"V:|[tableView]"
                          options:0
                          metrics:nil
                          views:views];
  
  // Only one constraint in this set. Save it for later removal if required
  NSLayoutConstraint *newTopConstraint = [constraints firstObject];
  self.tableViewTop = newTopConstraint;
  
  [self.view addConstraints:constraints];
}

// Update the title field and the picker wheels with the current settings
- (void)updateEditView {
  // The selected movie
  MovieTitle *aTitle = _titles[selectedRow];
  // Place its title in the text field
	self.titleField.text = aTitle.movieTitle;
	
  // Also update the picker with its genre
	[self.genrePicker selectRow:aTitle.movieGenre inComponent:0 animated:YES];
}

// See if the title exists in the movie list. Used by newMovieTitle
- (BOOL)titleExists:(NSString *)aTitle {
	// Fast enumerate through the movie list
  for (MovieTitle *movie in _titles) {
    // Does this title exist?
		if ([movie.movieTitle isEqualToString:aTitle]) {
      // Yes, return true
			return TRUE;
    }
  }
  return FALSE;
}

// Create a unique (localized) movie title. Used by addNewMovie and textFieldDidEndEditing
- (NSString *)newMovieTitle {
	// Place holder movie title. Should localize...
	//NSString *newTitle = NSLocalizedString(@"NewMovieTitle", @"Add a new title to the movie list");
	NSString *newTitle = @"New title";
	
	// Add an index to the new title if it is not unique
	NSUInteger newMovieIndex = 0;
	NSString *nextTitle = newTitle;
	
	// Create a unique new title indexed 1->n
	while ([self titleExists:nextTitle]) {
		newMovieIndex++;
		// Append the index to the base new title
		nextTitle = [newTitle stringByAppendingFormat:@" %lu", (unsigned long)newMovieIndex];
	}
	return nextTitle;
}

// Create a movie object from the edit view parameters. Used by textFieldDidEndEditing
- (MovieTitle *)createMovieWithTitle:(NSString *)title {
  // Get the selected genre from the picker
  NSInteger pickedGenre = [self.genrePicker selectedRowInComponent:0];
  
  // Create the movie object with title and genre
  MovieTitle *newMovie = [MovieTitle movieTitle:title withGenre:pickedGenre];
  
  return newMovie;
}

// Add a new movie title and update the movie list. Always called during editing
- (IBAction)addNewMovie:(id)sender {
	// Create a unique new localized title
	NSString *newTitle = [self newMovieTitle];
	
  // Make a new movie object and add it to the list
  MovieTitle *newMovie = [self createMovieWithTitle:newTitle];
  [_titles addObject:newMovie];
  
  // Sort the movie titles array alphabetically using a block sort
  [_titles sortUsingComparator:^(MovieTitle *title1, MovieTitle *title2) {
    return [title1.movieTitle localizedCaseInsensitiveCompare:title2.movieTitle];
  }];
  
  // Find the index of the new/changed title in the sorted list
  NSUInteger index = [self indexForTitle:newTitle];
  // This is the new row selection
  selectedRow = index;
  
  // Let the detail view know about the new title
  MovieTitle *movie = _titles[index];
  self.detailViewController.detailItem = movie;
  
  // Update the text field and the picker
  [self updateEditView];
  
	// Make sure the new title is selected during editing
	[self performSelector:@selector(selectEditedTitle) withObject:nil afterDelay:0.0];
	
	[self.tableView reloadData];
	
	// Open the text field for editing and show the keyboard
	[self.titleField becomeFirstResponder];
}

// Make sure the table view indicates the selected movie with a check mark
// Called by viewDidAppear and didSelectRowAtIndexPath
- (void)checkSelectedTitle {
	// List all row paths corresponding to titles (only after viewDidAppear)
  NSArray *indexPaths = [self.tableView indexPathsForVisibleRows];
  
	// Loop through all the titles
	for (NSIndexPath *aPath in indexPaths) {
		// Get a row index
		NSUInteger aRow = aPath.row;
    
		// Is this the selected title?
		if (aRow == selectedRow) {
			// Yes, check the title at this path
			[[self.tableView cellForRowAtIndexPath:aPath] setAccessoryType:UITableViewCellAccessoryCheckmark];
		} else {
			// No, uncheck this title
			[[self.tableView cellForRowAtIndexPath:aPath] setAccessoryType:UITableViewCellAccessoryNone];
		}
	}
}

// During editing, find the table view row with the selected title and select the row
// Called by showAccessoryView, textFieldDidEndEditing, commitEditingStyle, and addNewGame
- (void)selectEditedTitle {
	// List all row paths corresponding to movie titles
	NSArray *indexPaths = [self.tableView indexPathsForVisibleRows];
	
	// Loop through the movies until the one to be selected is found
	for (NSIndexPath *aPath in indexPaths) {
		// Get a row index
		NSUInteger aRow = aPath.row;
    
		// Is this the selected title?
		if (aRow == selectedRow) {
			// Yes, select the title at this path
			[self.tableView selectRowAtIndexPath:aPath animated:YES scrollPosition:UITableViewScrollPositionNone];
      
			break;
		}
	}
}

@end
